(***************************************************)
(* Office ToolBar Components                       *)
(* Version 1.0                                     *)
(***************************************************)
(*                                                 *)
(* (c) 1998-99 Evgesoft                            *)
(* All right reserved.                             *)
(*                                                 *)
(***************************************************)
(* Internet: dev@evgesoft.com                      *)
(* WWW:                                            *)
(*   http://www.evgesoft.com                       *)
(***************************************************)

unit msofficepanel;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls,
  Dialogs, StdCtrls, ExtCtrls, Buttons, AppBar, Menus, ComCtrls,
  Forms, op_common, op_wave, op_listbox;

const
  LinesCount = 64; // Color Lines in Gradient
  TitleHeight = 16; // Height of Title

  StdNum  = 102; // Number of exist panel
  
type

  // Panel Button

  TopButton = class(TSpeedButton)
  private
    FStore: boolean;
  public
    constructor Create(AOwner: TComponent); override;
  published
  end;

  TopSeparator = class(TLabel)
  private
    FStore: boolean;
  public
    constructor Create(AOwner: TComponent); override;
  published
  end;

  TopPanel = class;

  // Panel Title

  TopTitle = class(TCustomControl)
  private
    FSysMenuBtn: TImage;
    FPanel: TopPanel; // TopPanel object
    FSysMenu: TPopupMenu;
    FOnSysMenuDblClick: TNotifyEvent;
    // Called when the AppBar receives a WM_NCHITTEST message
    procedure OnNcHitTest (var Msg: TWMNCHitTest); message WM_NCHITTEST;
    procedure DoSysMenuDblClick(Sender: TObject);
    procedure DoSysMenuClick(Sender: TObject);
    function GetGlyph: TBitmap;
    procedure SetGlyph(Value: TBitmap);
  protected
    procedure Notification(AComponent: TComponent;
      Operation: TOperation); override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Paint; override;
  published
    property Font;
    property Glyph: TBitmap read GetGlyph write SetGlyph;
    property SysMenu: TPopupMenu read FSysMenu
      write FSysMenu;
    property OnSysMenuDblClick: TNotifyEvent read FOnSysMenuDblClick
      write FOnSysMenuDblClick;
  end;

  // Panel Caption

  TopCaption = class(TLabel)
  private
    // Called when the AppBar receives a WM_NCHITTEST message
    procedure OnNcHitTest (var Msg: TWMNCHitTest); message WM_NCHITTEST;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Paint; override;
  published
  end;

  // Panel Tabs

  TOnDropFiles = procedure (Sender: TObject; Point: TPoint;
    Files: TStrings) of object;

  TBackStyle = (bsStandard, bsColor, bsGradient, bsBitmap);

  TopTab = class(TCustomPanel)
  private
    FBtn: TSpeedButton;
    FCaption: TopCaption;
    FIndex: Integer;
    FAutoInsert: boolean;
    FBackStyle: TBackStyle;
    FBackFile: string;
    FBackBitmap: TBitmap;
    FStore: boolean; // Store in Registry (Run-Time)
    FPanel: TopPanel;

    FOnDropFiles: TOnDropFiles;
    procedure AddButton(FileName: string);
    procedure WMDropFiles(var Msg: TMessage); message WM_DROPFILES;

    function GetGlyph: Tbitmap;
    procedure SetGlyph(Value: TBitmap);
    procedure SetBackBitmap(Value: TBitmap);
    procedure SetBackStyle(Value: TBackStyle);

    procedure DoBtnClick(Sender: TObject);
    procedure DoButtonsClick(Sender: TObject);

    procedure DrawLine(Canvas: TCanvas; Num: Integer; Col: TColor; Vert: boolean);

    procedure WMEraseBkGnd(var Msg: TWMEraseBkgnd); message WM_EraseBkgnd;

    // Aligning Controls
    procedure Aligning;
    // Called when the AppBar receives a WM_NCHITTEST message
    procedure OnNcHitTest (var Msg: TWMNCHitTest); message WM_NCHITTEST;
  protected
    procedure ReadState(Reader: TReader); override;
    function GetClientRect: TRect; override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Loaded; override;
    procedure SetFocus; override;
    procedure Paint; override;
  published
    property AutoInsert: boolean read FAutoInsert
      write FAutoInsert; // Auto Insert Button after Drop File
    property BackStyle: TBackStyle read FBackStyle
      write SetBackStyle;
    property BackBitmap: TBitmap read FBackBitmap
      write SetBackBitmap;
    property Caption;
    property Index: Integer read FIndex;
    property Color;
    property Glyph: TBitmap read GetGlyph write SetGlyph;
    property Font;
    property OnDropFiles: TOnDropFiles read FOnDropFiles
      write FOnDropFiles;
  end;

  TopTabStyle = (tsStandard, tsOwnerDraw);

  TopMeasureTabEvent = procedure(Sender: TObject; Index: Integer;
    var TabWidth: Integer) of object;
  TopDrawTabEvent = procedure(Sender: TObject; TabCanvas: TCanvas; R: TRect;
    Index: Integer; Selected: Boolean) of object;
  TopTabChangeEvent = procedure(Sender: TObject; NewTab: Integer;
    var AllowChange: Boolean) of object;

  // Main Office Panel

  TopPanel = class(TCustomPanel)
  private
    FTabs: TList;
    FActiveTab: TopTab;
    FTitle: TopTitle;
    FAnimation: boolean;
    FSmallCaption: string;
    FAutoHide: boolean;
    FAlwaysOnTop: boolean;
    FSound: TWave;
    function GetPageCount: Integer;
    function GetTabs(Index: integer): TopTab;
    procedure SetActiveTab(Value: TopTab);
    procedure SetTitle(Value: TopTitle);
    procedure SetAutoHide(Value: boolean);
    procedure SetAlwaysOnTop(Value: boolean);
    procedure SetSound(Value: TWave);

    function isVertical: boolean;
    procedure Aligning;
    procedure PlaySound;

    // Called when the AppBar receives a WM_NCHITTEST message
    procedure OnNcHitTest (var Msg: TWMNCHitTest); message WM_NCHITTEST;
  protected
    procedure Notification(AComponent: TComponent;
      Operation: TOperation); override;
    function GetClientRect: TRect; override;
    procedure AlignControls(AControl: TControl; var Rect: TRect); override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Loaded; override;
    procedure Paint; override;
    // Add/Delete Tab
    procedure InsertTab(Tab: TopTab);
    procedure DeleteTab(Tab: TopTab);
    // Save and Load Setting from/to Registry
    procedure SaveSettings;
    procedure LoadSettings;
    // Add Button
    function InsertButton(ATab: TopTab): TopButton;
    function InsertSeparator(ATab: TopTab): TopSeparator;
    // WM_Resize
    procedure WMSize(var Msg: TWMSize); message WM_SIZE;
  published
    property ActiveTab: TopTab read FActiveTab write SetActiveTab;
    property Animation: boolean read FAnimation write FAnimation;
    property AutoHide: boolean read FAutoHide write SetAutoHide;
    property AlwaysOnTop: boolean read FAlwaysOnTop write SetAlwaysOnTop;
    property Font;
    property SmallCaption: string read FSmallCaption
      write FSmallCaption;
    property ShowHint;
    property Sound: TWave read FSound write SetSound;
    property TabCount: Integer read GetPageCount;
    property Tabs[Index: Integer]: TopTab read GetTabs;
    property Title: TopTitle read FTitle write SetTitle;
  end;

// Panel Customize form

  TfrmCustom = class(TForm)
    Pages: TPageControl;
    tsView: TTabSheet;
    tsButtons: TTabSheet;
    tsPanels: TTabSheet;
    btnOK: TBitBtn;
    btnCancel: TBitBtn;
    GroupBox1: TGroupBox;
    GroupBox2: TGroupBox;
    Label1: TLabel;
    cbPanel: TComboBox;
    cbShowHint: TCheckBox;
    cbAlwaysOnTop: TCheckBox;
    cbAutoHide: TCheckBox;
    Panel2: TPanel;
    rbColor: TRadioButton;
    rbBitmap: TRadioButton;
    rbStandard: TRadioButton;
    GroupBox3: TGroupBox;
    Label2: TLabel;
    Panel1: TPanel;
    pbColor: TPaintBox;
    btnChangeColor: TBitBtn;
    GroupBox4: TGroupBox;
    btnChangeBitmap: TBitBtn;
    cbGradient: TCheckBox;
    imBitmap: TImage;
    dlgChangeColor: TColorDialog;
    Label3: TLabel;
    btnAddPanel: TBitBtn;
    btnDeletePanel: TBitBtn;
    btnRenamePanel: TBitBtn;
    btnUpPanel: TSpeedButton;
    btnDownPanel: TSpeedButton;
    Label4: TLabel;
    cbPanel1: TComboBox;
    Label5: TLabel;
    btnAddFile: TBitBtn;
    btnAddFolder: TBitBtn;
    btnSeparator: TBitBtn;
    btnDelete: TBitBtn;
    OpenFile: TOpenDialog;
    cbAutoRun: TCheckBox;
    clbButtons: TopCheckListBox;
    clbPanels: TopCheckListBox;
    dlgChangeBitmap: TOpenDialog;
    procedure cbPanelChange(Sender: TObject);
    procedure btnOKClick(Sender: TObject);
    procedure btnChangeColorClick(Sender: TObject);
    procedure btnChangeBitmapClick(Sender: TObject);
    procedure pbColorPaint(Sender: TObject);
    procedure cbGradientClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure btnAddPanelClick(Sender: TObject);
    procedure btnDeletePanelClick(Sender: TObject);
    procedure btnRenamePanelClick(Sender: TObject);
    procedure btnUpPanelClick(Sender: TObject);
    procedure btnDownPanelClick(Sender: TObject);
    procedure cbPanel1Change(Sender: TObject);
    procedure rbColorClick(Sender: TObject);
    procedure rbBitmapClick(Sender: TObject);
    procedure rbStandardClick(Sender: TObject);
    procedure btnAddFileClick(Sender: TObject);
    procedure btnSeparatorClick(Sender: TObject);
    procedure btnDeleteClick(Sender: TObject);
    procedure clbPanelsClickCheck(Sender: TObject);
    procedure clbButtonsClickCheck(Sender: TObject);
    procedure btnAddFolderClick(Sender: TObject);
    procedure PagesChange(Sender: TObject);
  private
    { Private declarations }
    FPanel: TopPanel;
    imFileName: string;
    procedure DrawLine(Canvas: TCanvas; Num: Integer; Col: TColor);
    procedure LoadSettings;
  public
    { Public declarations }
    procedure Customize(APanel: TopPanel);
  end;

// Panel customize Component
  TopCustom = class(TComponent)
  private
    FPanel: TopPanel;
  protected
    procedure Notification(AComponent: TComponent;
      Operation: TOperation); override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Execute;
  published
    property Panel: TopPanel read FPanel write FPanel;
  end;

var
  frmCustom: TfrmCustom;

implementation {=======================================================}

{$R *.DFM}
{$R msofficepanel.res}

uses Registry, ShellAPI, MMSystem, Folder;

var
  bmpFolder: TBitmap;

{ TopPanel }

var
  Edge: TAppBarEdge;

constructor TopPanel.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FTabs := TList.Create;

  Width := 150;
  Height := 36;
  BevelInner := bvRaised;
  BevelOuter := bvNone;
  BorderWidth := 1;
  If Owner is TAppBar Then
    Edge := (Owner as TAppBar).Edge;

  FSound := TWave.Create;
end;

destructor TopPanel.Destroy;
begin
  FSound.Free;
  
  FTabs.Free;
  inherited Destroy;
end;

procedure TopPanel.Loaded;
begin
  inherited Loaded;
end;

procedure TopPanel.Paint;
var
  Rect: TRect;
begin
  Rect := Classes.Rect(0, 0, Width, Height);
  DrawEdge(Canvas.Handle, Rect, BDR_RAISEDOUTER, BF_RECT);
end;

procedure TopPanel.Notification(AComponent: TComponent;
      Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  if (Operation = opRemove) and (AComponent = FTitle) then
    FTitle := nil;
end;

procedure TopPanel.SetTitle(Value: TopTitle);
begin
  FTitle := Value;
  If FTitle <> nil Then
    FTitle.FPanel := Self;
end;

procedure TopPanel.PlaySound;
begin
  If csDesigning in ComponentState Then Exit;
  If (FSound <> nil) and (FSound.WaveData.Memory <> nil) Then
    sndPlaySound(FSound.WaveData.Memory, SND_MEMORY or SND_ASYNC);
end;

function TopPanel.isVertical: boolean;
begin
  If csDesigning in ComponentState Then
  begin
    Result := false;
    Exit;
  end;
  If (Owner is TAppBar) and ((Owner as TAppBar).Edge in [abeTop, abeBottom, abeFloat])
  then
    Result := false
  else
    Result := true;
end;

function TopPanel.GetClientRect: TRect;
begin
  Result := Rect(1, 1, Width-2, Height-2);
end;

procedure TopPanel.WMSize(var Msg: TWMSize);
begin
  inherited ;
  // Align Title and Self
  If not (csDesigning in ComponentState) and (Owner is TAppBar) and
     (FTitle <> nil) Then
  begin
    Edge := (Owner as TAppBar).Edge;
    If Edge in [abeTop, abeBottom] then
    begin
      // Change Title Align
      FTitle.Width := TitleHeight;
      FTitle.Height := (Owner as TAppBar).ClientHeight;
      SetBounds(TitleHeight, 0, (Owner as TAppBar).ClientWidth - TitleHeight,
        (Owner as TAppBar).ClientHeight);
    end
    else
    begin
      // Change Title Align
      FTitle.Height := TitleHeight;
      FTitle.Width := (Owner as TAppBar).ClientWidth;
      SetBounds(0, TitleHeight, (Owner as TAppBar).ClientWidth,
        (Owner as TAppBar).ClientHeight - TitleHeight);
    end;
  end;
  Aligning;
end;

procedure TopPanel.AlignControls(AControl: TControl; var Rect: TRect);
begin
  Aligning;
end;

procedure TopPanel.Aligning;
var
  i: integer;
  P1, P2: integer;
begin
  If FActiveTab = nil Then Exit;
  P1 := 1;
  If isVertical Then
    P2 := ClientHeight
  else
    P2 := ClientWidth;
  If FActiveTab.Index > 0 Then
    for i := 0 to FActiveTab.Index-1 do
      with TopTab(FTabs[i]) do
      begin
        If isVertical Then
        begin
          Top := 1+i*Self.ClientWidth;
          Left := 1;
          Width := Self.ClientWidth;
          Height := Self.ClientWidth;
        end
        else
        begin
          Top := 1;
          Left := 1+i*Self.ClientHeight;
          Width := Self.ClientHeight;
          Height := Self.ClientHeight;
        end;
        Inc(P1, Width);
        Aligning;
      end;
  If FTabs.Count > 1 Then
    for i := FActiveTab.Index+1 to FTabs.Count-1 do
      with TopTab(FTabs[i]) do
      begin
        If isVertical Then
        begin
          Top := 1+Self.ClientHeight-(FTabs.Count-i)*Self.ClientWidth;
          Left := 1;
          Width := Self.ClientWidth;
          Height := Self.ClientWidth;
        end
        else
        begin
          Top := 1;
          Left := 1+Self.ClientWidth-(FTabs.Count-i)*Self.ClientHeight;
          Width := Self.ClientHeight;
          Height := Self.ClientHeight;
        end;
        Dec(P2, Width);
        Aligning;
      end;
  If isVertical Then
  begin
    FActiveTab.Top := P1;
    FActiveTab.Height := P2-P1;
    FActiveTab.Left := 1;
    FActiveTab.Width := ClientWidth;
  end
  else
  begin
    FActiveTab.Top := 1;
    FActiveTab.Height := ClientHeight;
    FActiveTab.Left := P1;
    FActiveTab.Width := P2-P1;
  end;
  FActiveTab.Aligning;
end;

procedure TopPanel.SaveSettings;
var
  reg : TRegistry;
  i, j, Count, BCount: integer;
begin
  // Create a TRegistry object
  reg := TRegistry.Create;
  // Set the RootKey
  reg.RootKey := AB_DEF_ROOT_KEY;
  // Save Settings
  WriteBool('', rsShowHints, ShowHint);
  // Save Tabs Settings
  Count := 0;
  for i := 0 to FTabs.Count-1 do
    with TopTab(FTabs[i]) do
    begin
      // Save Tab Caption
      If FStore Then
      begin
        // Additional Tab
        WriteString(rsTabs, 'Tab'+IntToStr(Count), Caption);
        Inc(Count);
      end
      else
        // Std Tab
        WriteString(rsTabs, 'Tab'+IntToStr(StdNum+i), Caption);
      // Save Tab Settings
      WriteBool(rsTabs+'\'+Caption, rsVisible, Visible);
      WriteInteger(rsTabs+'\'+Caption, rsBackStyle, Integer(FBackStyle));
      WriteString(rsTabs+'\'+Caption, rsBackFile, FBackFile);
      WriteInteger(rsTabs+'\'+Caption, rsColor, Integer(Color));
      // Save Buttons
      BCount := 0;
      for j := 0 to ControlCount-1 do
      begin
        if (Controls[j] is TopButton) and ((Controls[j] as TopButton).FStore) Then
        begin
          // Save Button Settings
          WriteString(rsTabs+'\'+Caption, rsButton+IntToStr(BCount),
            (Controls[j] as TopButton).Hint);
          Inc(BCount);
        end;
        if (Controls[j] is TopSeparator) and ((Controls[j] as TopSeparator).FStore) Then
        begin
          // Save Separator Settings
          WriteString(rsTabs+'\'+Caption, rsButton+IntToStr(BCount),
            '-');
          Inc(BCount);
        end;
      end;
      WriteInteger(rsTabs+'\'+Caption, rsBCount, BCount);
    end;
  WriteInteger(rsTabs, rsCount, Count);
  WriteInteger('', rsActiveTab, FActiveTab.FIndex);
  // Free the TRegistry object
  reg.Destroy
end;

procedure TopPanel.LoadSettings;
  procedure LoadButton(Tab: TopTab; Caption: string);
  var
    BCount, j: integer;
  begin
    // Load Buttons
    BCount := ReadInteger(rsTabs+'\'+Caption, rsBCount, 0);
    If BCount > 0 Then
      for j := 0 to BCount-1 do
        Tab.AddButton(ReadString(rsTabs+'\'+Caption, rsButton+IntToStr(j), ''));
  end;
var
  reg : TRegistry;
  i, Count: integer;
  Tab: TopTab;
begin
  // Create a TRegistry object
  reg := TRegistry.Create;
  // Set the RootKey
  reg.RootKey := AB_DEF_ROOT_KEY;
  ShowHint := ReadBool('', rsShowHints, ShowHint);
  // Load Settings for Exist Tab
  for i := 0 to FTabs.Count-1 do
    with TopTab(FTabs[i]) do
    begin
      // Load Tab Caption
      Caption := ReadString(rsTabs, 'Tab'+IntToStr(StdNum+i), Caption);
      // Load Tab Settings
      Visible := ReadBool(rsTabs+'\'+Caption, rsVisible, Visible);
      BackStyle := TBackStyle(ReadInteger(rsTabs+'\'+Caption, rsBackStyle, Integer(BackStyle)));
      FBackFile := ReadString(rsTabs+'\'+Caption, rsBackFile, FBackFile);
      If FBackFile <> '' Then
        FBackBitmap.LoadFromFile(FBackFile);
      Color := TColor(ReadInteger(rsTabs+'\'+Caption, rsColor, Integer(Color)));
      // Load Buttons
      LoadButton(TopTab(FTabs[i]), Caption);
    end;
  // Read New Tab
  Count := ReadInteger(rsTabs, rsCount, 0);
  If Count > 0 Then
    for i := 0 to Count-1 do
    begin
      Tab := TopTab.Create(Self);
      with Tab do
      begin
        // Add new tab
        Parent := SElf;
        Glyph := bmpFolder;
        FStore := true;
        If Self.Width > Self.Height Then
        begin
          Left := Self.Width
        end
        else
        begin
          Top := Self.Height;
          FBtn.Height := FBtn.Width;
        end;
        InsertTab(Tab);
        DragAcceptFiles(Tab.Handle, true);
        Aligning;
        // Load Tab Caption
        Caption := ReadString(rsTabs, 'Tab'+IntToStr(i), Caption);
        // Load Tab Settings
        Visible := ReadBool(rsTabs+'\'+Caption, rsVisible, Visible);
        FBackStyle := TBackStyle(ReadInteger(rsTabs+'\'+Caption, rsBackStyle, Integer(FBackStyle)));
        Color := TColor(ReadInteger(rsTabs+'\'+Caption, rsColor, Integer(Color)));
        // Load Buttons
        LoadButton(Tab, Caption);
      end;
    end;
  ActiveTab := FTabs[ReadInteger('', rsActiveTab, FActiveTab.FIndex)];
  // Free the TRegistry object
  reg.Destroy;
end;

function TopPanel.GetPageCount: Integer;
begin
  Result := FTabs.Count;
end;

function TopPanel.GetTabs(Index: integer): TopTab;
begin
  Result := FTabs[Index];
end;

function TopPanel.InsertButton(ATab: TopTab): TopButton;
begin
  Result := nil;
  If ATab = nil Then Exit;
  Result := TopButton.Create(Owner);
  Result.Parent := ATab;
  If not isVertical then
    Result.Left := Width
  else
    Result.Top := Height;
  Result.Left := Width;
  Result.Width := Result.Height;
  ATab.Aligning;
end;

function TopPanel.InsertSeparator(ATab: TopTab): TopSeparator;
begin
  Result := nil;
  If ATab = nil Then Exit;
  Result := TopSeparator.Create(Owner);
  Result.Parent := ATab;
  If not isVertical then
    Result.Left := Width
  else
    Result.Top := Height;
  Result.Left := Width;
  Result.Width := Result.Height;
  ATab.Aligning;
end;

procedure TopPanel.InsertTab(Tab: TopTab);
begin
  Tab.FIndex := FTabs.Count;
  If Tab.FPanel = nil Then Tab.FPanel := Self;
  FTabs.Add(Tab);
end;

procedure TopPanel.DeleteTab(Tab: TopTab);
begin
  FTabs.Delete(Tab.Index);
end;

procedure TopPanel.SetAlwaysOnTop(Value: boolean);
begin
  If Value <> FAlwaysOnTop Then
  begin
    FAlwaysOnTop := Value;
  end;
  If not (csDesigning in ComponentState) and (Owner is TAppBar) Then
    with Owner as TAppBar do
    begin
      Application.ProcessMessages;
      AlwaysOnTop := FAlwaysOnTop;
      UpdateBar;
    end;
end;

procedure TopPanel.SetAutoHide(Value: boolean);
begin
  If Value <> FAutoHide Then
  begin
    FAutoHide := Value;
  end;
  If not (csDesigning in ComponentState) and (Owner is TAppBar) Then
    with Owner as TAppBar do
    begin
      Application.ProcessMessages;
      AutoHide := FAutoHide;
      UpdateBar;
    end;
end;

procedure TopPanel.SetActiveTab(Value: TopTab);
begin
  FActiveTab := Value;
  Aligning;
end;

procedure TopPanel.OnNcHitTest (var Msg: TWMNCHitTest);
begin
  inherited ;
  If not (csDesigning in ComponentState) Then
    Msg.Result := HTTRANSPARENT;
end;

procedure TopPanel.SetSound(Value: TWave);
begin
  { copy data from source }
  FSound.Assign(Value);
end;

{ TopTab }

constructor TopTab.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FBtn := TSpeedButton.Create(Self);
  FBtn.Parent := Self;
  FBtn.GroupIndex := 100+random(1000);
  FBtn.AllowAllUp := true;
  FBtn.OnClick := DoBtnClick;

  FCaption := TopCaption.Create(Self);
  FCaption.Parent := Self;

  FAutoInsert := true;
  FBackStyle := bsGradient;

  FBackBitmap := TBitmap.Create;
  FStore := false;

  Width := 100;
  Height := 32;
  Caption := '';
  BorderWidth := 3;
  BevelInner := bvNone;
  BevelOuter := bvLowered;
end;

procedure TopTab.Loaded;
begin
  inherited Loaded;
  Canvas.Font := Font;
  FCaption.Width := Canvas.TextWidth(Caption) + 20;
  If not(csDesigning in ComponentState) Then
    DragAcceptFiles(Handle, true);
  FBtn.Width := FBtn.Height;
  If (FBtn.Glyph <> nil) and (FBtn.Glyph.Width > FBtn.Glyph.Height) Then
    FBtn.NumGlyphs := FBtn.Glyph.Width div FBtn.Glyph.Height;
end;

destructor TopTab.Destroy;
begin
  FBackBitmap.Free;
  FBtn.Free;
  inherited Destroy;
end;

procedure TopTab.DrawLine(Canvas: TCanvas; Num: Integer; Col: TColor; Vert: boolean);
begin
  Canvas.Brush.Color := Col;
{  If Vert Then
    Canvas.Rectangle(0, ((Height div 2) div LinesCount +1)*Num,
        Width, ((Height div 2) div LinesCount+1)*(Num+1)+1)
  else
    Canvas.Rectangle(((width div 2) div LinesCount +1)*Num, 0,
        ((width div 2) div LinesCount +1)*(Num+1)+1, Height);}
  If Vert Then
    Canvas.Rectangle(0, ((Height - 10)  div LinesCount +1)*Num,
        Width, ((Height - 10) div LinesCount+1)*(Num+1)+1)
  else
    Canvas.Rectangle(((width - 10) div LinesCount +1)*Num, 0,
        ((width - 10) div LinesCount +1)*(Num+1)+1, Height);
end;

type
  TRGB = record
    R,G,B, Res: byte;
  end;

function TopTab.GetClientRect: TRect;
begin
  Result := Rect(3, 3, Width-6, Height-6);
end;

procedure TopTab.Aligning;
var
  i, Cx, Cy, W, H: integer;
begin
  Cx := BorderWidth;
  Cy := BorderWidth;
  for i := 0 to ControlCount-1 do
  begin
    Controls[i].Left := Cx;
    Controls[i].Top := Cy;
    If not FPanel.isVertical Then
      H := ClientHeight
    else
      H := ClientWidth;

    W := H;
    If Controls[i] is TopCaption Then // Control is Caption
      W := Canvas.TextWidth(Caption)+20;
    If Controls[i] is TopSeparator Then // Control is Separator
      W := 10;

    If not FPanel.isVertical Then
    begin
      Inc(Cx, W);
      Controls[i].Width := W;
      Controls[i].Height := H;
    end
    else
    begin
      Inc(Cy, W);
      Controls[i].Width := H;
      Controls[i].Height := W;
    end;

    If Self <> FPanel.FActiveTab Then FBtn.Down := false
    else FBtn.Down := true;
  end;
  Canvas.Font := Font;
end;

procedure TopTab.Paint;
var
  Rect: TRect;
  i: integer;
  C: Longint;
  RGB, RGB1, RGB2: TRGB;
  Rrange, Grange, Brange: real;
  Rpos, Gpos, Bpos: real;
  B: TBitmap;
  XCnt, YCnt, X, Y: Integer;
  BevelSize: Integer;
begin
  Rect := Classes.Rect(0, 0, Width, Height);
  DrawEdge(Canvas.Handle, Rect, BDR_RAISEDINNER, BF_RECT);
  If FPanel.FActiveTab = Self Then
  begin
    B := TBitmap.Create;
    B.Width := Width-2;
    B.Height := Height-2;
    with B do
    begin
      case FBackStyle of
        bsStandard:
          begin
            Canvas.Pen.Style := psClear;
            Canvas.Brush.Style := bsSolid;
            Canvas.Brush.Color := FPanel.Color;
            Canvas.Rectangle(0, 0, Width+1, Height+1);
          end;
        bsColor:
          begin
            Canvas.Pen.Style := psClear;
            Canvas.Brush.Style := bsSolid;
            Canvas.Brush.Color := Color;
            Canvas.Rectangle(0, 0, Width+1, Height+1);
          end;
        bsGradient: // Draw Back Gradient
          begin
            Canvas.Pen.Style := psClear;
            Canvas.Brush.Style := bsSolid;
            Canvas.Brush.Color := FPanel.Color;
            Canvas.Rectangle(0, 0, Width+1, Height+1);
            RGB1 := TRGB(ColorToRGB(Color));
            RGB2 := TRGB(ColorToRGB(FPanel.Color));
            Rrange := (RGB2.R - RGB1.R) / LinesCount;
            Grange := (RGB2.G - RGB1.G) / LinesCount;
            Brange := (RGB2.B - RGB1.B) / LinesCount;
            RGB := RGB1;
            Rpos := RGB.R;
            Gpos := RGB.G;
            Bpos := RGB.B;
            for i := 0 to LinesCount do begin
              RGB.R := Round(Rpos);
              RGB.G := Round(Gpos);
              RGB.B := Round(Bpos);
              Move(RGB, C, 4);
              If Self.Width > Self.Height Then
                DrawLine(Canvas, I, C, false)
              else
                DrawLine(Canvas, I, C, true);
              If RGB.R < 255 Then Rpos := Rpos + Rrange;
              If RGB.G < 255 Then Gpos := Gpos + Grange;
              If RGB.B < 255 Then Bpos := Bpos + Brange;
            end;
          end;
        bsBitmap: // Draw Back Bitmap
          If FBackBitmap.Width > 0 Then
          begin
            Rect := ClientRect;
            InflateRect(Rect, 5, 5);
            OffSetRect(Rect, 1, 1);
            BevelSize := 1;
            InflateRect(Rect, -BevelSize, -BevelSize);
            IntersectClipRect(Canvas.Handle, Rect.Left, Rect.Top,
              Rect.Right -Rect.Left, Rect.Bottom-Rect.Top);
            XCnt := (ClientWidth - 2 * BevelSize) div FBackBitmap.Width;
            YCnt := (ClientHeight - 2 * BevelSize) div FBackBitmap.Height;
            for X := 0 to XCnt do
              for Y := 0 to YCnt do
                Canvas.Draw(Rect.Left + X * FBackBitmap.Width,
                  Rect.Top + Y * FBackBitmap.Height,
                  FBackBitmap);
          end;
      end;
      Canvas.Font := Self.Font;
      Canvas.Brush.Style := bsClear;
      If Width > Height Then
        // Normal
        Canvas.TextOut(FBtn.Width + 10, (Height -
          Canvas.TextHeight(Caption)) div 2, Caption)
      else
        // Vertical Text
        If (Owner as TControl).Left < Screen.Width div 2 Then
          // Left
          AngleTextOut(Canvas, (Width - Canvas.TextHeight(Caption)) div 2,
            FBtn.Height + 10 + Canvas.TextWidth(Caption) div 2, 90, Caption)
        else
          // Right
          AngleTextOut(Canvas, (Width - Canvas.TextHeight(Caption)) div 2 +
            Canvas.TextHeight(Caption), FBtn.Height + 10 -
            Canvas.TextWidth(Caption) div 2, 270, Caption)
    end;
    Canvas.Draw(1, 1, B);
  end
  else
  begin
    Canvas.Pen.Style := psClear;
    Canvas.Brush.Color := Color;
    Canvas.Rectangle(1, 1, Width, Height);
  end;
end;

procedure TopTab.WMEraseBkGnd(var Msg: TWMEraseBkgnd);
begin
  If csDesigning in ComponentState Then
    inherited;
end;

procedure TopTab.DoBtnClick(Sender: TObject);
begin
  If FPanel.ActiveTab <> Self Then
  begin
    FPanel.PlaySound;
    SetFocus
  end
  else
    FBtn.Down := true;
end;

procedure TopTab.SetFocus;
const
  Dx = 2;
var
  i: integer;
  OldTab: TopTab;
begin
  inherited SetFocus;
  with FPanel do
  begin
    OldTab := FActiveTab;
    FActiveTab := Self;
    If FAnimation Then
    begin
      // Animation
      If not isVertical Then // Horizontal
      begin
        OldTab.Width := Self.Height;
        If OldTab.FIndex > FIndex Then
          OldTab.Left := OldTab.Left + OldTab.Width - OldTab.Height;
        for i := 1 to OldTab.Width div Dx do
        begin
          If OldTab.FIndex > FIndex Then
            MoveWindow(Self.Handle, Self.Left, Self.Top, Self.Width+i*dx,
              Self.Height, true)
          else
            MoveWindow(Self.Handle, Self.Left-i*dx, Self.Top, Self.Width,
              Self.Height, true);
          If i mod Dx = 0 Then
            Application.ProcessMessages;
        end;
      end
      else
      begin // Verical
        OldTab.Height := Self.Width;
        If OldTab.FIndex > FIndex Then
          OldTab.Top := OldTab.Top + OldTab.Height - OldTab.Width;
        for i := 1 to OldTab.Height div Dx do
        begin
          If OldTab.FIndex > FIndex Then
            MoveWindow(Self.Handle, Self.Left, Self.Top, Self.Width,
              Self.Height+i*dx, true)
          else
            MoveWindow(Self.Handle, Self.Left, Self.Top-i*dx, Self.Width,
              Self.Height, true);
          If i mod Dx = 0 Then
            Application.ProcessMessages;
        end;
      end;
    end;
    Aligning;
    // End Animation
  end;
end;

procedure TopTab.ReadState(Reader: TReader);
begin
  inherited ReadState(Reader);
  if Reader.Parent is TopPanel then
  begin
    FPanel := TopPanel(Reader.Parent);
    FPanel.InsertTab(Self);
    If not FPanel.isVertical Then
      Left := FPanel.Width
    else
      Top := FPanel.Width;
  end;
  If FPanel.isVertical Then
    FBtn.Height := FPanel.ClientWidth
  else
    FBtn.Height := FPanel.ClientHeight;
  FBtn.Width := FBtn.Height;
end;

procedure TopTab.SetBackStyle(Value: TBackStyle);
begin
  If Value <> FBackStyle Then
  begin
    FBackStyle := Value;
    Invalidate;
  end;
end;

procedure TopTab.SetBackBitmap(Value: TBitmap);
begin
  FBackBitmap.Assign(Value);
  If Value <> nil Then
    FBackStyle := bsBitmap;
end;

function TopTab.GetGlyph: Tbitmap;
begin
  Result := FBtn.Glyph;
end;

procedure TopTab.SetGlyph(Value: TBitmap);
begin
  FBtn.Glyph := Value;
end;

procedure TopTab.DoButtonsClick(Sender: TObject);
begin
  with Sender as TSpeedButton do
    ShellExecute(0, 'open', PChar(hint), nil, nil, SW_SHOWNORMAL);
end;

procedure TopTab.WMDropFiles(var Msg: TMessage);
var
  DD: HDrop;
  i, Count: integer;
  FileName: string;
  FStrings: TStrings;
  P: TPoint;
begin
  DD := Msg.wParam;
  Count := DragQueryFile(DD, -1, PChar(FileName), Length(FileName));
  DragQueryPoint(DD, P);
  FStrings := TStringList.Create;
  for i := 0 to Count-1 do
  begin
    SetLength(FileName, $FF);
    DragQueryFile(DD, i, PChar(FileName), Length(FileName));
    FStrings.Add(FileName);
    AddButton(FileName);
  end;
  If Assigned(OnDropFiles) Then OnDropFiles(Self, P, FStrings);
  FStrings.Free;
  DragFinish(DD);
end;

function GetFileIcon(FileName: string): TIcon;
var
  Info: TSHFileInfo;
begin
  Result := TIcon.Create;
  SHGetFileInfo(PChar(FileName), 0, Info, SizeOf(Info),
    SHGFI_ICON or SHGFI_SMALLICON);
  Result.Handle := Info.hIcon;
end;

procedure DrawSmallIcon(Canvas: TCanvas; X, Y: integer; Icon: TIcon; MaskColor: TColor);
begin
  with Canvas do
  begin
    Brush.Color := MaskColor;
    DrawIconEx(Handle, X, Y, Icon.Handle, 16, 16, 0,
      Brush.Handle, DI_IMAGE);
  end;
end;

procedure TopTab.AddButton(FileName: string);
var
  Icon: TIcon;
  Bitmap: Tbitmap;
  S: TopSeparator;
  B: TopButton;
begin
  If FileName = '' Then Exit;
  If FileName = '-' Then
  begin
    // Insert Separator
    S := TopSeparator.Create(Self);
    S.Parent := Self;
    S.FStore := true;
    If not FPanel.isVertical Then
    begin
      S.Left := FPanel.Width;
      S.Width := 10;
    end
    else
    begin
      S.Top := FPanel.Height;
      S.Height := 10;
    end;
    Exit;
  end;
  B := FPanel.InsertButton(Self);
  with B do
  begin
{      S := ExtractFileName(FileName);
      System.Delete(S, Length(S)-Length(ExtractFileExt(FileName))+1,
        Length(ExtractFileExt(FileName)));}
    Hint := FileName;
    FStore := true;
    ShowHint := true;
    Icon := GetFileIcon(FileName);
    Bitmap := TBitmap.Create;
    Bitmap.Width := 16;
    Bitmap.Height := 16;
    DrawSmallIcon(Bitmap.Canvas, 0, 0, Icon, clOlive);
    Glyph := Bitmap;
    Bitmap.Free;
    Icon.Free;
    OnClick := DoButtonsClick;
  end;
  Aligning;
end;

procedure TopTab.OnNcHitTest (var Msg: TWMNCHitTest);
begin
  inherited ;
  If not (csDesigning in ComponentState) Then
    Msg.Result := HTTRANSPARENT;
end;

{ TopCaption }

constructor TopCaption.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  Transparent := true;
  ParentFont := true;
  AutoSize := false;
  Caption := '';
end;

destructor TopCaption.Destroy;
begin
  inherited Destroy;
end;

procedure TopCaption.Paint;
begin
  Exit;
end;

procedure TopCaption.OnNcHitTest (var Msg: TWMNCHitTest);
begin
  inherited ;
  If not (csDesigning in ComponentState) Then
    Msg.Result := HTTRANSPARENT;
end;

{ TopTitle }

constructor TopTitle.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);

  FSysMenuBtn := TImage.Create(Self);
  FSysMenuBtn.Parent := Self;
  FSysMenuBtn.Left := 3;
  FSysMenuBtn.Top := 3;
  FSysMenuBtn.Width := TitleHeight - 4;
  FSysMenuBtn.Height := TitleHeight - 4;
  FSysMenuBtn.OnDblClick := DoSysMenuDblClick;
  FSysMenuBtn.OnClick := DoSysMenuClick;

  Color := clActiveCaption;
  Width := 100;
  Height := TitleHeight;
end;

destructor TopTitle.Destroy;
begin
  FSysMenuBtn.Free;
  inherited Destroy;
end;

procedure TopTitle.Notification(AComponent: TComponent;
      Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  if (Operation = opRemove) and (AComponent = FSysMenu) then
    FSysMenu := nil;
end;

procedure TopTitle.DoSysMenuDblClick(Sender: TObject);
begin
  If Assigned(FOnSysMenuDblClick) Then
    FOnSysMenuDblClick(Self);
end;

procedure TopTitle.DoSysMenuClick(Sender: TObject);
var
  P: TPoint;
begin
  If FSysMenu <> nil Then
  begin
    P := ClientToScreen(Point(2, Height+2));
    FSysMenu.Popup(P.X, P.Y);
  end;
end;

function TopTitle.GetGlyph: TBitmap;
begin
  Result := FSysMenuBtn.Picture.Bitmap;
end;

procedure TopTitle.SetGlyph(Value: TBitmap);
begin
  FSysMenuBtn.Picture.Bitmap := Value;
end;

procedure TopTitle.Paint;
var
  R: TRect;
begin
  If Application.Active Then
    Color := clActiveCaption
  else
    Color := clInActiveCaption;
  inherited ;
  IF Owner is TForm Then
    with Canvas do
    begin
      Font := Self.Font;
      Font.Color := clCaptionText;
      Brush.Style := bsClear;
      If (Owner is TAppBar) and ((Owner as TAppBar).Edge <> abeFloat)
      Then
      begin
        // Draw Small Caption
        Canvas.Font.Size := Canvas.Font.Size - 2;
        Canvas.Font.Style := Canvas.Font.Style + [fsBold]; 
        If FPanel <> nil then
        begin
          If Width > Height Then
            TextOut(TitleHeight+3, 4, FPanel.FSmallCaption)
          else
            AngleTextOut(Self.Canvas, 4, TitleHeight + 5 +
              TextWidth(FPanel.FSmallCaption) div 2, 90,
              FPanel.FSmallCaption)
        end
      end
      else
        // Normal Caption
        TextOut(TitleHeight+4, 2, (Owner as TForm).Caption);
      R := ClientRect;
      If Width > Height Then
        Inc(R.Bottom, 2)
      else
        Inc(R.Right, 2);
      DrawEdge(Handle, R, BDR_RAISEDOUTER, BF_RECT);
      InflateRect(R, -1, -1);
      DrawEdge(Handle, R, BDR_RAISEDINNER, BF_RECT);
      // Draw Bevel between SysMenuBtn and Caption
      If Width > Height Then
        R := Rect(FSysMenuBtn.Width+4, 2, FSysMenuBtn.Width+6,
          FSysMenuBtn.Height+4)
      else
        R := Rect(2, FSysMenuBtn.Height+4, FSysMenuBtn.Width+4,
          FSysMenuBtn.Height+6);
      DrawEdge(Handle, R, BDR_SUNKENOUTER, BF_RECT);
    end;
end;

procedure TopTitle.OnNcHitTest (var Msg: TWMNCHitTest);
begin
  inherited ;
  If not (csDesigning in ComponentState) Then
    Msg.Result := HTTRANSPARENT;
end;

// TopCustom

constructor TopCustom.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
end;

destructor TopCustom.Destroy;
begin
  inherited Destroy;
end;

procedure TopCustom.Notification(AComponent: TComponent;
    Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  if (Operation = opRemove) and (AComponent = FPanel) then
    FPanel := nil;
end;

procedure TopCustom.Execute;
begin
  If FPanel <> nil Then
  begin
    frmCustom := TfrmCustom.Create(Self);
    frmCustom.Customize(FPanel);
    frmCustom.Free;
  end;
end;

{ TopButton }

constructor Topbutton.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FStore := false;
end;

{ TopSeparator }

constructor TopSeparator.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FStore := false;
  Transparent := true;
end;

// Customize Form

var
  OldTab: integer; // Save Old Tab

procedure TfrmCustom.LoadSettings;
var
  i: integer;
begin
  with FPanel do
  // Synhronize Settings with Controls
  begin
    cbAutoRun.Checked := ReadBool(rsAutoRun, rsAutoRun, true);
    cbPanel.Items.Clear;
    cbPanel1.Items.Clear;
    clbPanels.Items.Clear;
    for i := 0 to FTabs.Count-1 do
    begin
      // to View Page
      cbPanel.Items.Add(TopTab(FTabs[i]).Caption);
      // to Panels Page
      cbPanel1.Items.Add(TopTab(FTabs[i]).Caption);
      clbPanels.Items.Add(TopTab(FTabs[i]).Caption);
      clbPanels.Checked[i] := TopTab(FTabs[i]).Visible;
    end;
    cbPanel.ItemIndex := 0;
    cbPanel1.OnChange(Self);
    cbShowHint.checked := ShowHint;
    cbAlwaysOnTop.checked := AlwaysOnTop;
    cbAutoHide.checked := AutoHide;
    // Disable Always On Top
    AlwaysOnTop := false;
  end;
end;

procedure TfrmCustom.Customize(APanel: TopPanel);
begin
  FPanel := APanel;
  LoadSettings;
  OldTab := cbPanel.ItemIndex;
  cbPanelChange(Self);
  ShowModal;
  cbPanelChange(Self);
end;

procedure TfrmCustom.rbColorClick(Sender: TObject);
begin
  // Color Click
  btnChangeColor.Enabled := true;
  btnChangeBitmap.Enabled := false;
  cbGradient.Checked := true;
end;

procedure TfrmCustom.rbBitmapClick(Sender: TObject);
begin
  // Bitmap Click
  btnChangeColor.Enabled := false;
  btnChangeBitmap.Enabled := true;
end;

procedure TfrmCustom.rbStandardClick(Sender: TObject);
begin
  // Standard Click
  btnChangeColor.Enabled := false;
  btnChangeBitmap.Enabled := false;
end;

procedure TfrmCustom.cbPanelChange(Sender: TObject);
begin
  // Save Old Tab
  with TopTab(FPanel.Tabs[OldTab]) do
  begin
    If rbStandard.Checked Then
      BackStyle := bsStandard;
    If rbColor.Checked Then
    begin
      If cbGradient.Checked Then
        BackStyle := bsGradient
      else
        BackStyle := bsColor;
      Color := pbColor.Color;
    end;
    If rbBitmap.Checked Then
    begin
      BackStyle := bsBitmap;
      BackBitmap := imBitmap.Picture.Bitmap;
      If imFileName <> '' Then
        FBackFile := imFileName;
    end;
  end;
  // Change Cur Tab
  btnChangeColor.Enabled := false;
  btnChangeBitmap.Enabled := false;
  with TopTab(FPanel.FTabs[cbPanel.ItemIndex]) do
  begin
    case FBackStyle of
      bsStandard: rbStandard.Checked := true;
      bsColor:
      begin
        pbColor.Color := Color;
        rbColor.Checked := true;
        cbGradient.Checked := false;
        btnChangeColor.Enabled := true;
      end;
      bsGradient:
      begin
        pbColor.Color := Color;
        rbColor.Checked := true;
        cbGradient.Checked := true;
        btnChangeColor.Enabled := true;
      end;
      bsBitmap:
      begin
        btnChangeBitmap.Enabled := true;
        rbBitmap.Checked := true;
        imBitmap.Picture.Bitmap := BackBitmap;
      end;
    end;
  end;
  OldTab := cbPanel.ItemIndex;
end;

procedure TfrmCustom.btnOKClick(Sender: TObject);
var
  Reg: TRegistry;
begin
  // Save Setting
  with FPanel do
  // Synhronize Settings with Controls
  begin
    ShowHint := cbShowHint.Checked;
    AlwaysOnTop := cbAlwaysOnTop.checked;
    AutoHide := cbAutoHide.checked;
    WriteBool(rsAutoRun, rsAutoRun, cbAutoRun.Checked);
    // Delete/Add key from/to Windows\Run
    Reg := TRegistry.Create;
    try
      with Reg do
      begin
        RootKey := HKEY_LOCAL_MACHINE;
        OpenKey('\SOFTWARE\Microsoft\Windows\CurrentVersion\Run', False);
        If not cbAutoRun.Checked Then
          DeleteValue(ExtractFileName(ParamStr(0)))
        else
          WriteString(ExtractFileName(ParamStr(0)), ParamStr(0))
      end;
    finally
      Reg.Free;
    end;
  end;
  FPanel.SaveSettings;
end;

procedure TfrmCustom.btnChangeColorClick(Sender: TObject);
begin
  dlgChangeColor.Color := pbColor.Color;
  If dlgChangeColor.Execute Then
    pbColor.Color := dlgChangeColor.Color;
end;

procedure TfrmCustom.btnChangeBitmapClick(Sender: TObject);
begin
  If dlgChangeBitmap.Execute Then
  begin
    imBitmap.Picture.Bitmap.LoadFromFile(dlgChangeBitmap.FileName);
    imFileName := dlgChangeBitmap.FileName;
    TopTab(FPanel.FTabs[cbPanel.ItemIndex]).FBackFile := imFileName;
  end;
end;

procedure TfrmCustom.DrawLine(Canvas: TCanvas; Num: Integer; Col: TColor);
begin
  Canvas.Brush.Color := Col;
  with pbColor do
    Canvas.Rectangle((width div LinesCount +1)*Num, 0,
        (width div LinesCount +1)*(Num+1)+1, Height);
end;

procedure TfrmCustom.pbColorPaint(Sender: TObject);
var
  i: integer;
  C: Longint;
  RGB, RGB1, RGB2: TRGB;
  Rrange, Grange, Brange: real;
  Rpos, Gpos, Bpos: real;
begin
  // Draw Color
  if pbColor.Enabled Then
    with pbColor.Canvas do
      If cbGradient.Checked Then
      begin
        // Draw Gradient
        Pen.Style := psClear;
        Brush.Style := bsSolid;
        Brush.Color := (pbColor.Parent as TPanel).Color;
        Rectangle(0, 0, Width+1, Height+1);
        RGB1 := TRGB(ColorToRGB(pbColor.Color));
        RGB2 := TRGB(ColorToRGB((pbColor.Parent as TPanel).Color));
        Rrange := (RGB2.R - RGB1.R) / LinesCount;
        Grange := (RGB2.G - RGB1.G) / LinesCount;
        Brange := (RGB2.B - RGB1.B) / LinesCount;
        RGB := RGB1;
        Rpos := RGB.R;
        Gpos := RGB.G;
        Bpos := RGB.B;
        for i := 0 to LinesCount do begin
          RGB.R := Round(Rpos);
          RGB.G := Round(Gpos);
          RGB.B := Round(Bpos);
          Move(RGB, C, 4);
          DrawLine(pbColor.Canvas, I, C);
          If RGB.R < 255 Then Rpos := Rpos + Rrange;
          If RGB.G < 255 Then Gpos := Gpos + Grange;
          If RGB.B < 255 Then Bpos := Bpos + Brange;
        end;
      end
      else
      begin
        // Draw Color
        Pen.Style := psClear;
        Brush.Style := bsSolid;
        Brush.Color := pbColor.Color;
        Rectangle(0, 0, Width+1, Height+1);
      end
  else
    with pbColor.Canvas do
      begin
        // Draw Color
        Pen.Style := psClear;
        Brush.Style := bsSolid;
        Brush.Color := Color;
        Rectangle(0, 0, Width+1, Height+1);
      end;
end;

procedure TfrmCustom.cbGradientClick(Sender: TObject);
begin
  Panel1.Invalidate;
end;

procedure TfrmCustom.FormCreate(Sender: TObject);
begin
  Pages.ActivePage := tsView;
end;

procedure TfrmCustom.btnAddPanelClick(Sender: TObject);
var
  Tab: TopTab;
  PName: string;
begin
  // Add Panel
  PName := 'new panel';
  If InputQuery('Add new panel', 'Panel name:', PName) Then
  begin
    Tab := TopTab.Create(FPanel);
    Tab.Parent := FPanel;
    Tab.Caption := PName;
    Tab.Glyph := bmpFolder;
    Tab.FStore := true;
    If FPanel.Width > FPanel.Height Then
    begin
      Tab.Left := FPanel.Width
    end
    else
    begin
      Tab.Top := FPanel.Height;
      Tab.FBtn.Height := Tab.FBtn.Width;
    end;
    FPanel.InsertTab(Tab);
    DragAcceptFiles(Tab.Handle, true);
    FPanel.Aligning;
  end;
  LoadSettings;
end;

procedure TfrmCustom.btnDeletePanelClick(Sender: TObject);
var
  DelIndex, i: integer;
begin
  // Delete Panel
  DelIndex := clbPanels.ItemIndex;
  If TopTab(FPanel.FTabs[DelIndex]).FIndex+1 < FPanel.FTabs.Count Then
    for i := TopTab(FPanel.FTabs[DelIndex]).FIndex+1 to FPanel.FTabs.Count-1 do
      TopTab(FPanel.FTabs[i]).FIndex :=
        TopTab(FPanel.FTabs[i]).FIndex - 1;
  If FPanel.FActiveTab = TopTab(FPanel.FTabs[DelIndex]) Then
  begin
    If DelIndex > 0 Then
      FPanel.ActiveTab := TopTab(FPanel.FTabs[DelIndex-1]);
    If DelIndex+1 < FPanel.FTabs.Count Then
      FPanel.ActiveTab := TopTab(FPanel.FTabs[DelIndex+1]);
  end;
  FPanel.RemoveControl(TopTab(FPanel.FTabs[DelIndex]));
  clbPanels.Items.Delete(DelIndex);
  FPanel.FTabs.Delete(DelIndex);
  FPanel.Aligning;
end;

procedure TfrmCustom.btnRenamePanelClick(Sender: TObject);
var
  PName: string;
begin
  // Rename Panel
  PName := TopTab(FPanel.FTabs[clbPanels.ItemIndex]).Caption;
  If InputQuery('Rename panel', 'New panel name:', PName) Then
  begin
    TopTab(FPanel.FTabs[clbPanels.ItemIndex]).Caption := PName;
  end;
  LoadSettings;
end;

procedure TfrmCustom.btnUpPanelClick(Sender: TObject);
var
  T: integer; // temp variable
  Tab1, Tab2: TopTab;
begin
  If clbPanels.ItemIndex > 0 Then
  begin
    // Exchange in Panel
    FPanel.FTabs.Exchange(clbPanels.ItemIndex-1, clbPanels.ItemIndex);
    Tab1 := TopTab(FPanel.FTabs[clbPanels.ItemIndex-1]);
    Tab2 := TopTab(FPanel.FTabs[clbPanels.ItemIndex]);
    T := Tab1.FIndex; Tab1.FIndex := Tab2.FIndex; Tab2.FIndex := T;
    T := Tab1.Left; Tab1.Left := Tab2.Left; Tab2.Left := T;
    T := Tab1.Top; Tab1.Top := Tab2.Top; Tab2.Top := T;
    FPanel.Aligning;
    // Panels List
    LoadSettings;
  end;
end;

procedure TfrmCustom.btnDownPanelClick(Sender: TObject);
var
  T: integer; // temp variable
  Tab1, Tab2: TopTab;
begin
  If clbPanels.ItemIndex < clbPanels.Items.Count-1 Then
  begin
    // Exchange in Panel
    FPanel.FTabs.Exchange(clbPanels.ItemIndex+1, clbPanels.ItemIndex);
    Tab1 := TopTab(FPanel.FTabs[clbPanels.ItemIndex+1]);
    Tab2 := TopTab(FPanel.FTabs[clbPanels.ItemIndex]);
    T := Tab1.FIndex; Tab1.FIndex := Tab2.FIndex; Tab2.FIndex := T;
    T := Tab1.Left; Tab1.Left := Tab2.Left; Tab2.Left := T;
    T := Tab1.Top; Tab1.Top := Tab2.Top; Tab2.Top := T;
    FPanel.Aligning;
    LoadSettings;
  end;
end;

procedure TfrmCustom.clbPanelsClickCheck(Sender: TObject);
begin
  // Click Check
  TopTab(FPanel.FTabs[clbPanels.ItemIndex]).Visible :=
    clbPanels.Checked[clbPanels.ItemIndex];
  If not (FPanel.FActiveTab.Visible) then
  begin
    If clbPanels.ItemIndex > 0 Then
      FPanel.ActiveTab := TopTab(FPanel.FTabs[clbPanels.ItemIndex-1]);
    If clbPanels.ItemIndex+1 < FPanel.FTabs.Count Then
      FPanel.ActiveTab := TopTab(FPanel.FTabs[clbPanels.ItemIndex+1]);
  end;
  FPanel.Aligning;
end;

procedure TfrmCustom.cbPanel1Change(Sender: TObject);
var
  i, Old: integer;
begin
  // Buttons Panel Change
  Old := clbButtons.ItemIndex;
  clbButtons.Items.Clear;
  If cbPanel1.ItemIndex < 0 Then cbPanel1.ItemIndex := 0;
  with TopTab(FPanel.FTabs[cbPanel1.ItemIndex]) do
    for i := 0 to ControlCount-1 do
    begin
      If Controls[i] is TopButton Then
      begin
        clbButtons.Items.Add((Controls[i] as TopButton).Hint);
        clbButtons.Checked[clbButtons.Items.Count-1] :=
          (Controls[i] as TopButton).Visible;
      end;
      If Controls[i] is TopSeparator Then
      begin
        clbButtons.Items.Add('---Separator---');
        clbButtons.Checked[clbButtons.Items.Count-1] :=
          (Controls[i] as TopSeparator).Visible;
      end;
    end;
  If Old > 0 Then clbButtons.ItemIndex := Old
  else clbButtons.ItemIndex := 0;
end;

procedure TfrmCustom.btnAddFileClick(Sender: TObject);
begin
  // Add File
  If OpenFile.Execute Then
    with TopTab(FPanel.FTabs[cbPanel1.ItemIndex]) do
    begin
      AddButton(OpenFile.FileName);
      cbPanel1Change(Self);
    end;
end;

procedure TfrmCustom.btnAddFolderClick(Sender: TObject);
var
  FName: string;
begin
  // Add Folder
  FName := GetFolder;
  If FName <> '' Then
    with TopTab(FPanel.FTabs[cbPanel1.ItemIndex]) do
    begin
      AddButton(FName);
      cbPanel1Change(Self);
    end;
end;

procedure TfrmCustom.btnSeparatorClick(Sender: TObject);
begin
  // Ad separator
  with TopTab(FPanel.FTabs[cbPanel1.ItemIndex]) do
  begin
    AddButton('-');
    cbPanel1Change(Self);
  end;
end;

procedure TfrmCustom.btnDeleteClick(Sender: TObject);
var
  i: integer;
begin
  // Delete Button
  with TopTab(FPanel.FTabs[cbPanel1.ItemIndex]) do
  begin
    for i := 0 to ControlCount-1 do
    begin
      If (Controls[i] is TopButton) and
         ((Controls[i] as TopButton).Hint = clbButtons.Items[clbButtons.ItemIndex])
        Then
        begin
          RemoveControl(Controls[i]);
          Break;
        end;
      If (Controls[i] is TopSeparator) and
         (i - 2 = clbButtons.ItemIndex)
        Then
        begin
          RemoveControl(Controls[i]);
          Break;
        end;
    end;
    cbPanel1Change(Self);
  end;
end;

procedure TfrmCustom.clbButtonsClickCheck(Sender: TObject);
var
  i: integer;
begin
  // Check Click
  with TopTab(FPanel.FTabs[cbPanel1.ItemIndex]) do
  begin
    for i := 0 to ControlCount-1 do
    begin
      If (Controls[i] is TopButton) and
         ((Controls[i] as TopButton).Hint = clbButtons.Items[clbButtons.ItemIndex])
        Then
        begin
          Controls[i].Visible := clbButtons.Checked[clbButtons.ItemIndex];
          Break;
        end;
      If (Controls[i] is TopSeparator) and
         (i - 2 = clbButtons.ItemIndex)
        Then
        begin
          Controls[i].Visible := clbButtons.Checked[clbButtons.ItemIndex];
          Break;
        end;
    end;
    cbPanel1Change(Self);
  end;
end;

procedure TfrmCustom.PagesChange(Sender: TObject);
begin
  LoadSettings;
end;

initialization
  bmpfolder := TBitmap.Create;
  bmpFolder.Handle := LoadBitmap(HInstance, 'FOLDER');
finalization
  bmpFolder.Free;
end.
