(***************************************************)
(* Office ToolBar Components                       *)
(* Version 1.0                                     *)
(* Registration Unit                               *)
(***************************************************)
(*                                                 *)
(* (c) 1998-99 Evgesoft                            *)
(* All right reserved.                             *)
(*                                                 *)
(***************************************************)
(* Internet: dev@evgesoft.com                      *)
(* WWW:                                            *)
(*   http://www.evgesoft.com                       *)
(***************************************************)

unit opreg;

interface

uses Classes, Menus, dialogs;

procedure Register;

implementation {===========================================================}

uses Controls, DsgnIntf, msofficepanel, buttons, Forms,
     op_Common, op_TrayIcon, op_NoTask, op_wave, op_ListBox;

type

  TOfficePanelEditor = class(TDefaultEditor)
    procedure ExecuteVerb(Index: Integer); override;
    function GetVerb(Index: Integer): string; override;
    function GetVerbCount: Integer; override;
  end;

{ TopPanelEditor ============================================== }

const
  OfficePanelVerbs: array[0..2] of String =
    ('Insert tab', 'Insert button', 'Insert Separator');

procedure TOfficePanelEditor.ExecuteVerb(Index: Integer);
var
  Panel: TopPanel;
  Tab: TopTab;
  Btn: TopButton;
  Sep: TopSeparator;
  Designer: TFormDesigner;
begin
  Panel := nil;
  If Component is TopPanel Then
    Panel := Component as TopPanel;
  If Component is TopTab Then
    Panel := (Component as TopTab).Parent as TopPanel;
  If Panel = nil Then Exit;
  if Panel <> nil then
  begin
    Designer := Self.Designer;
    Tab := nil;
    case Index of
    0: begin
        Tab := TopTab.Create(Designer.Form);
        try
          Tab.Name := Designer.UniqueName(TopTab.ClassName);
          Tab.Parent := Panel;
        except
          Tab.Free;
          raise;
        end;
        If Tab <> nil Then
        begin
          Panel.InsertTab(Tab);
          Panel.ActiveTab := Tab;
        end;
      end;
    1: If Panel.ActiveTab <> nil Then
       begin
         Btn := Panel.InsertButton(Panel.ActiveTab);
         If Btn <> nil Then
           Btn.Name := Designer.UniqueName(Btn.ClassName);
       end;
    2: If Panel.ActiveTab <> nil Then
       begin
         Sep := Panel.InsertSeparator(Panel.ActiveTab);
         If Sep <> nil Then
           Sep.Name := Designer.UniqueName(Sep.ClassName);
         Sep.Caption := '';
       end;
    end;
    Designer.SelectComponent(Tab);
    Designer.Modified;
  end;
end;

function TOfficePanelEditor.GetVerb(Index: Integer): string;
begin
  Result := OfficePanelVerbs[Index];
end;

function TOfficePanelEditor.GetVerbCount: Integer;
begin
  Result := High(OfficePanelVerbs) + 1;
end;

{ ==========================================================}

type

  TSoundProperty = class(TPropertyEditor)
  public
    procedure Edit; override;
    function GetAttributes: TPropertyAttributes; override;
    function GetValue: string; override;
    procedure SetValue(const Value: string); override;
  end;

{ TSoundProperty ====================================== }

procedure TSoundProperty.Edit;
var
  Comp: TopPanel;
  OpenDialog: TOpenDialog;
begin
  try
    Comp := TopPanel(GetComponent(0));
    OpenDialog := TOpenDialog.Create(Application);
    OpenDialog.FileName := '*.wav';
    OpenDialog.Filter := 'WAV Files|*.wav|All files|*.*';
    If OpenDialog.Execute then
    begin
      Comp.Sound.LoadFromFile(OpenDialog.FileName);
      Comp.Sound.Modified := true;
    end;
    OpenDialog.Free;
  finally
  end;
end;

function TSoundProperty.GetAttributes: TPropertyAttributes;
begin
  Result := [paDialog];
end;

function TSoundProperty.GetValue: string;
begin
  Result := '(WAV Sound)';
end;

procedure TSoundProperty.SetValue(const Value: string);
begin
  if Value = '' then SetOrdValue(0);
end;

{ Registration }

procedure Register;
begin
  RegisterComponents('Office Panel',
  [
    TopPanel,
    TopButton,
    TopSeparator,
    TopTitle,
    TopCustom,
    TopCheckListBox
  ]);

  RegisterComponents('Office Panel',
  [
    TopNoTask,
    TopTrayIcon
  ]);

  RegisterClass(TopTab);
  RegisterClass(TWave);

  RegisterComponentEditor(TopPanel, TOfficePanelEditor);
  RegisterComponentEditor(TopTab, TOfficePanelEditor);

  RegisterPropertyEditor(TypeInfo(TWave), nil,'Sound', TSoundProperty);
end;

end.
